#! /usr/bin/python3
import gnupg, time, string, itertools, mimetypes
import multiprocessing

# GPG symmetric decryption bruteforcing in Python
# -----------------------------------------------
# photo.jpg : original image
# photo.jpg.asc : encrypted using $ gpg -ca photo.jpg
# Hint passphrase is "chat"
#
# Set your input below
##################################################
filepath = 'photo.jpg.asc'
characters = string.ascii_lowercase
password_length = 4
proc_lot = 500
nb_proc=6
test_solo_mode = False #Set this to True to know what will be your speed in sequential mode (It's only a test. Machines have multiple cores now so it's not very usefull to not use them)
##################################################

def open_file(filepath):
  if mimetypes.guess_type(filepath)[0] == "application/pgp-encrypted":
    mode = "rb"
  else :
    mode = "r"
  f = open(filepath, mode)
  data = f.read()
  f.close()
  return data

def take(nb_elements, nb_slots):
  words = [["".join(a) for a in itertools.islice(generator, nb_elements)] for i in range(nb_slots)]
  return words

def test_solo(generator, nb_tests):
  i = 0
  t_start = time.time()
  print("performing test_solo for %i tests" % nb_tests)
  for password in generator:
    password = "".join(password)
    if gpg.decrypt(data, passphrase=password).ok == True:
      print("possible password '%s' found" % password)
    i+=1
    if i == nb_tests:
      break
  print("time per test in solo : %.2e sec." % ((time.time()-t_start)/nb_tests))

def test_parallel(passwords):
  for password in passwords:
    if gpg.decrypt(data, passphrase=password).ok == True:
      print("possible password '%s' found" % password)

data = open_file(filepath)
print("Looking for valid passwords of %i characters" % password_length)
nb_combinations = len(characters)**password_length
print("Combinations to test: %i" % nb_combinations)
generator = itertools.product(characters, repeat=password_length)
print("Start testing...")
gpg = gnupg.GPG()
if test_solo_mode == True:
  test_solo(generator, proc_lot)
else:
  nb_password_computed = 0
  with multiprocessing.Pool(processes=nb_proc) as pool:
    while nb_password_computed < nb_combinations :
      t_start = time.time()
      n_start = nb_password_computed
      passwords_list = take(proc_lot, nb_proc)
      pool.map(test_parallel, passwords_list, 1)
      nb_password_computed += proc_lot*nb_proc
      nb_password_computed_by_step = nb_password_computed-n_start
      t_per_test = (time.time()-t_start)/nb_password_computed_by_step
      nb_remaining_tests = nb_combinations-nb_password_computed
      if nb_remaining_tests < 0:
        nb_remaining_tests = 0
      eta = nb_remaining_tests*t_per_test/3600
      print("time per test: %.2e sec. Remaining tests: %i. ETA to complete: %.2f h." % (t_per_test, nb_remaining_tests, eta))

